@@ -19,8 +19,11 @@
  *
  */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include "phonemgr-listener.h"
 
@@ -73,13 +76,15 @@ main (int argc, char **argv)
 	GError *err = NULL;
 	PhonemgrListener *listener;
 
+	setlocale (LC_ALL, "");
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
 	g_thread_init (NULL);
 	g_type_init ();
 	
-	listener = phonemgr_listener_new ();
-
-	if (!listener)
-		g_error ("Couldn't make listener");
+	listener = phonemgr_listener_new (TRUE);
 
 	g_signal_connect (G_OBJECT (listener), "message",
 			G_CALLBACK (message), (gpointer) listener);
@@ -90,6 +95,7 @@ main (int argc, char **argv)
 	g_signal_connect (G_OBJECT (listener), "battery",
 			  G_CALLBACK (battery), (gpointer) listener);
 
+//	if (phonemgr_listener_connect (listener, "1", &err)) {
 	if (phonemgr_listener_connect (listener, "00:12:D2:79:B7:33", &err)) {
 //	if (phonemgr_listener_connect (listener, "/dev/rfcomm0", &err)) {
 		g_message ("Connected OK");
