@@ -69,6 +69,7 @@ struct _PhonemgrListener
 	PhonemgrState *phone_state;
 
 	char *driver;
+	char *own_number;
 
 	/* The previous call status */
 	gn_call_status prev_call_status;
@@ -80,9 +81,12 @@ struct _PhonemgrListener
 
 	guint connected : 1;
 	guint terminated : 1;
+	guint debug : 1;
 
 	/* Whether the driver supports GN_OP_OnSMS */
 	guint supports_sms_notif : 1;
+	/* Whether or not the driver supports GN_OP_GetPowersource */
+	guint supports_power_source : 1;
 };
 
 static void phonemgr_listener_class_init (PhonemgrListenerClass *klass);
@@ -101,10 +105,53 @@ enum {
 	LAST_SIGNAL
 };
 
+enum {
+	PROP_0,
+	PROP_DEBUG,
+};
+
 static int phonemgr_listener_signals[LAST_SIGNAL] = { 0 } ;
 
 G_DEFINE_TYPE (PhonemgrListener, phonemgr_listener, G_TYPE_OBJECT)
 
+static void
+phonemgr_listener_get_property (GObject *object,
+				 guint property_id,
+				 GValue *value,
+				 GParamSpec *pspec)
+{
+	PhonemgrListener *l;
+
+	l = PHONEMGR_LISTENER (object);
+
+	switch (property_id) {
+	case PROP_DEBUG:
+		g_value_set_boolean (value, l->debug != FALSE);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+static void
+phonemgr_listener_set_property (GObject *object,
+				guint property_id,
+				const GValue *value,
+				GParamSpec *pspec)
+{
+	PhonemgrListener *l;
+
+	l = PHONEMGR_LISTENER (object);
+
+	switch (property_id) {
+	case PROP_DEBUG:
+		l->debug = g_value_get_boolean (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
 static void
 phonemgr_listener_class_init (PhonemgrListenerClass *klass)
 {
@@ -113,6 +160,8 @@ phonemgr_listener_class_init (PhonemgrListenerClass *klass)
 	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
 	object_class = (GObjectClass*) klass;
+	object_class->set_property = phonemgr_listener_set_property;
+	object_class->get_property = phonemgr_listener_get_property;
 
 	phonemgr_listener_signals[MESSAGE_SIGNAL] =
 		g_signal_new ("message",
@@ -158,6 +207,11 @@ phonemgr_listener_class_init (PhonemgrListenerClass *klass)
 			      2,
 			      G_TYPE_INT, G_TYPE_BOOLEAN);
 
+	g_object_class_install_property (object_class,
+					 PROP_DEBUG,
+					 g_param_spec_boolean ("debug", NULL, NULL,
+							       FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	object_class->finalize = phonemgr_listener_finalize;
 
 	klass->message = NULL;
@@ -211,9 +265,24 @@ phonemgr_listener_emit_message (PhonemgrListener *l, gn_sms *message)
 	text = NULL;
 
 	/* The data should be in whatever the locale's encoding is,
+	 * if it doesn't fit the default GSM alphabet (roughly ISO8859-1)
 	 * as a design decision in gnokii */
 	origtext = (char *) message->user_data[0].u.text;
-	if (g_utf8_validate (origtext, -1, NULL) == FALSE) {
+
+	if (gn_char_def_alphabet (origtext)) {
+		GError *err = NULL;
+
+		text = g_convert (origtext, -1,
+				  "UTF-8", "ISO8859-1",
+				  NULL, NULL, &err);
+
+		if (err != NULL) {
+			g_warning ("Conversion error from GSM default alphabet: %d %s", err->code, err->message);
+			g_error_free (err);
+			g_free (text);
+			text = g_strdup (origtext);
+		}
+	} else if (g_utf8_validate (origtext, -1, NULL) == FALSE) {
 		GError *err = NULL;
 
 		text = g_locale_to_utf8 (origtext, -1, NULL, NULL, &err);
@@ -221,6 +290,7 @@ phonemgr_listener_emit_message (PhonemgrListener *l, gn_sms *message)
 		if (err != NULL) {
 			g_warning ("Conversion error: %d %s", err->code, err->message);
 			g_error_free (err);
+			g_free (text);
 			text = g_strdup (origtext);
 		}
 	}
@@ -240,9 +310,9 @@ phonemgr_listener_emit_message (PhonemgrListener *l, gn_sms *message)
 }
 
 PhonemgrListener *
-phonemgr_listener_new (void)
+phonemgr_listener_new (gboolean debug)
 {
-	return PHONEMGR_LISTENER (g_object_new (PHONEMGR_TYPE_LISTENER, NULL));
+	return PHONEMGR_LISTENER (g_object_new (PHONEMGR_TYPE_LISTENER, "debug", debug, NULL));
 }
 
 #ifndef DUMMY
@@ -252,7 +322,9 @@ phonemgr_listener_init (PhonemgrListener *l)
 	l->queue = g_async_queue_new ();
 	l->mutex = g_mutex_new ();
 	l->driver = NULL;
+	l->own_number = NULL;
 	l->batterylevel = 1;
+	l->supports_power_source = TRUE;
 	l->powersource = GN_PS_BATTERY;
 }
 
@@ -278,15 +350,15 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 {
 	int channel;
 
+	g_return_val_if_fail (PHONEMGR_IS_LISTENER (l), FALSE);
 	g_return_val_if_fail (l->connected == FALSE, FALSE);
 	g_return_val_if_fail (l->phone_state == NULL, FALSE);
 
 	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTING);
 
-
 	channel = -1;
 
-	if (phonemgr_utils_is_bluetooth (device) != FALSE) {
+	if (phonemgr_utils_address_is (device) == PHONEMGR_CONNECTION_BLUETOOTH) {
 		channel = phonemgr_utils_get_channel (device);
 		if (channel < 0) {
 			//FIXME
@@ -294,7 +366,7 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 		}
 	}
 
-	l->phone_state = phonemgr_utils_connect (device, NULL, channel, error);
+	l->phone_state = phonemgr_utils_connect (device, NULL, channel, l->debug, error);
 	if (l->phone_state == NULL) {
 		//FIXME
 		return FALSE;
@@ -307,10 +379,11 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 	}
 
 	/* Need a different driver? then reconnect */
-	if (strcmp (l->driver, PHONEMGR_DEFAULT_DRIVER) != 0) {
+	if (strcmp (l->driver, PHONEMGR_DEFAULT_DRIVER) != 0
+	    && strcmp (l->driver, PHONEMGR_DEFAULT_USB_DRIVER) != 0) {
 		phonemgr_utils_disconnect (l->phone_state);
 		phonemgr_utils_free (l->phone_state);
-		l->phone_state = phonemgr_utils_connect (device, l->driver, channel, error);
+		l->phone_state = phonemgr_utils_connect (device, l->driver, channel, l->debug, error);
 		if (l->phone_state == NULL) {
 			//FIXME
 			return FALSE;
@@ -323,7 +396,7 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTED);
 
 	l->thread = g_thread_create ((GThreadFunc) phonemgr_listener_thread,
-			l, TRUE, NULL);
+				     l, TRUE, NULL);
 
 	return l->connected;
 }
@@ -488,14 +561,20 @@ phonemgr_listener_battery_poll (PhonemgrListener *l)
 	(&l->phone_state->data)->power_source = &powersource;
 	(&l->phone_state->data)->battery_unit = &battery_unit;
 
-	if (gn_sm_functions(GN_OP_GetPowersource, &l->phone_state->data, &l->phone_state->state) != GN_ERR_NONE)
+	/* Some drivers will use the same function for battery level and power source, so optimise.
+	 * Make sure to get the battery level first, as most drivers implement only this one */
+	if (gn_sm_functions(GN_OP_GetBatteryLevel, &l->phone_state->data, &l->phone_state->state) != GN_ERR_NONE)
+		return;
+
+	if (powersource == -1 && l->supports_power_source != FALSE) {
+		if (gn_sm_functions(GN_OP_GetPowersource, &l->phone_state->data, &l->phone_state->state) != GN_ERR_NONE) {
+			g_message ("driver or phone doesn't support getting the power source");
+			l->supports_power_source = FALSE;
+			powersource = GN_PS_BATTERY;
+		}
+	} else if (l->supports_power_source == FALSE) {
 		powersource = GN_PS_BATTERY;
-
-	/* Some drivers will use the same function for battery level and power source, so optimise */
-	if (batterylevel == -1)
-		if (gn_sm_functions(GN_OP_GetBatteryLevel, &l->phone_state->data, &l->phone_state->state) != GN_ERR_NONE)
-			return;
-
+	}
 
 	if (batterylevel != l->batterylevel || powersource != l->powersource) {
 		AsyncSignal *signal;
@@ -503,9 +582,10 @@ phonemgr_listener_battery_poll (PhonemgrListener *l)
 		l->batterylevel = batterylevel;
 		l->powersource = powersource;
 
-		/* Probably not the best guess, but that's what xgnokii uses */
+		/* As mentioned by Daniele Forsi at:
+		 * http://thread.gmane.org/gmane.linux.drivers.gnokii/9329/focus=9331 */
 		if (battery_unit == GN_BU_Arbitrary)
-			batterylevel *= 25;
+			batterylevel = batterylevel * 100 / l->phone_state->state.driver.phone.max_battery_level;
 
 		signal = g_new0 (AsyncSignal, 1);
 		signal->type = BATTERY_SIGNAL;
@@ -519,6 +599,64 @@ phonemgr_listener_battery_poll (PhonemgrListener *l)
 
 }
 
+static void
+phonemgr_listener_get_own_details (PhonemgrListener *l)
+{
+	gn_memory_status memstat;
+	gn_phonebook_entry entry;
+	int count, start_entry, end_entry, num_entries;
+
+	start_entry = 1;
+	end_entry = num_entries = INT_MAX;
+
+	memstat.memory_type = gn_str2memory_type("ON");
+	l->phone_state->data.memory_status = &memstat;
+	if (gn_sm_functions(GN_OP_GetMemoryStatus, &l->phone_state->data, &l->phone_state->state) == GN_ERR_NONE) {
+		num_entries = memstat.used;
+		end_entry = memstat.used + memstat.free;
+	}
+
+	count = start_entry;
+	while (num_entries > 0 && count <= end_entry) {
+		gn_error error = GN_ERR_NONE;
+
+		memset(&entry, 0, sizeof(gn_phonebook_entry));
+		entry.memory_type = memstat.memory_type;
+		entry.location = count;
+
+		l->phone_state->data.phonebook_entry = &entry;
+		error = gn_sm_functions(GN_OP_ReadPhonebook, &l->phone_state->data, &l->phone_state->state);
+		if (error != GN_ERR_NONE && error != GN_ERR_EMPTYLOCATION)
+			break;
+		if (entry.empty != FALSE)
+			continue;
+		if (error == GN_ERR_NONE)
+			num_entries--;
+		count++;
+
+		if (!entry.subentries_count && entry.number) {
+			l->own_number = g_strdup (entry.number);
+		} else {
+			int i;
+
+			for (i = 0; i < entry.subentries_count; i++) {
+				if (entry.subentries[i].entry_type != GN_PHONEBOOK_ENTRY_Number)
+					continue;
+
+				l->own_number = g_strdup (entry.number);
+				break;
+			}
+		}
+
+		break;
+	}
+
+	if (l->own_number != NULL)
+		g_message ("Our own phone number is: %s", l->own_number);
+	else
+		g_message ("Couldn't get our own phone number");
+}
+
 static void
 phonemgr_listener_set_sms_notification (PhonemgrListener *l, gboolean state)
 {
@@ -529,9 +667,9 @@ phonemgr_listener_set_sms_notification (PhonemgrListener *l, gboolean state)
 		l->phone_state->data.callback_data = l;
 		if (gn_sm_functions (GN_OP_OnSMS, &l->phone_state->data, &l->phone_state->state) == GN_ERR_NONE) {
 			l->supports_sms_notif = TRUE;
-			g_message ("driver supports sms notifications");
+			g_message ("driver and phone support sms notifications");
 		} else {
-			g_message ("driver doesn't support sms notifications");
+			g_message ("driver or phone doesn't support sms notifications");
 		}
 	} else {
 		if (l->supports_sms_notif == FALSE)
@@ -565,6 +703,7 @@ static void
 phonemgr_listener_thread (PhonemgrListener *l)
 {
 	g_mutex_lock (l->mutex);
+	phonemgr_listener_get_own_details (l);
 	phonemgr_listener_set_sms_notification (l, TRUE);
 	phonemgr_listener_set_call_notification (l, TRUE);
 	g_mutex_unlock (l->mutex);
@@ -594,6 +733,7 @@ phonemgr_listener_thread (PhonemgrListener *l)
 void
 phonemgr_listener_disconnect (PhonemgrListener *l)
 {
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	g_return_if_fail (l->connected != FALSE);
 
 	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_DISCONNECTING);
@@ -603,6 +743,8 @@ phonemgr_listener_disconnect (PhonemgrListener *l)
 
 	g_free (l->driver);
 	l->driver = NULL;
+	g_free (l->own_number);
+	l->own_number = NULL;
 
 	l->connected = FALSE;
 	//FIXME more to kill?
@@ -616,12 +758,14 @@ phonemgr_listener_disconnect (PhonemgrListener *l)
 void
 phonemgr_listener_cancel_call (PhonemgrListener *l)
 {
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	//FIXME
 }
 
 void
 phonemgr_listener_answer_call (PhonemgrListener *l)
 {
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	//FIXME
 }
 
@@ -629,11 +773,12 @@ void
 phonemgr_listener_queue_message (PhonemgrListener *l,
 		const char *number, const char *message)
 {
-	char *mstr;
+	char *mstr, *iso;
 	GError *err = NULL;
 	gn_sms sms;
 	gn_error error;
 
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	g_return_if_fail (l->connected != FALSE);
 	g_return_if_fail (number != NULL);
 	g_return_if_fail (message != NULL);
@@ -643,20 +788,32 @@ phonemgr_listener_queue_message (PhonemgrListener *l,
 	gn_data_clear(&l->phone_state->data);
 	gn_sms_default_submit(&sms);
 
+	/* Default GSM alphabet is a subset of ISO8859-1,
+	 * so try that */
+	iso = g_convert (message, -1,
+			 "ISO8859-1", "UTF-8",
+			 NULL, NULL, &err);
+	if (err != NULL) {
+		g_clear_error (&err);
+		g_free (iso);
+		iso = NULL;
+	}
+
 	/* If the message contains characters not in the
-	 * default GSM alaphabet, we convert it to UCS-2 encoding instead */
-	if (gn_char_def_alphabet((unsigned char *) message)) {
-		mstr = g_strdup (message);
+	 * default GSM alaphabet, we pass it as UTF-8 encoding instead */
+	if (iso && gn_char_def_alphabet((unsigned char *) iso)) {
+		mstr = iso;
 		sms.dcs.u.general.alphabet = GN_SMS_DCS_DefaultAlphabet;
 	} else {
-		mstr = g_convert (message, strlen (message),
-				  "UCS-2", "UTF-8",
-				  NULL, NULL, &err);
+		g_free (iso);
+		/* gnokii will convert to UCS-2 itself, but we pass UTF-8 */
+		mstr = g_strdup (message);
 		sms.dcs.u.general.alphabet = GN_SMS_DCS_UCS2;
 	}
 
 	if (err != NULL) {
 		g_warning ("Conversion error: %d %s", err->code, err->message);
+		g_free (mstr);
 		g_error_free (err);
 		g_mutex_unlock (l->mutex);
 		return;
@@ -690,6 +847,9 @@ phonemgr_listener_queue_message (PhonemgrListener *l,
 	/* Actually send the message */
 	error = gn_sms_send (&l->phone_state->data, &l->phone_state->state);
 
+	/* Remove the reference to SMS */
+	l->phone_state->data.sms = NULL;
+
 	/* Unlock the phone */
 	g_mutex_unlock (l->mutex);
 }
@@ -703,6 +863,10 @@ phonemgr_listener_set_time (PhonemgrListener *l,
 	gn_error error;
 	PhoneMgrError perr;
 
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
+	g_return_if_fail (l->connected == FALSE);
+	g_return_if_fail (l->phone_state == NULL);
+
 	t = localtime(&time);
 	date.year = t->tm_year;
 	date.month = t->tm_mon + 1;
@@ -728,41 +892,15 @@ phonemgr_listener_set_time (PhonemgrListener *l,
 }
 
 gboolean
-phonemgr_listener_connected (PhonemgrListener *listener)
+phonemgr_listener_connected (PhonemgrListener *l)
 {
-	return listener->connected;
+	g_return_val_if_fail (PHONEMGR_IS_LISTENER (l), FALSE);
+
+	return l->connected;
 }
 
 #else /* !DUMMY */
 
-gboolean
-phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **err)
-{
-	g_message ("[DUMMY] connecting to %s", device);
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTING);
-	g_usleep (G_USEC_PER_SEC * 2);
-	l->connected = TRUE;
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTED);
-
-	return TRUE;
-}
-
-void
-phonemgr_listener_disconnect (PhonemgrListener *l)
-{
-	g_message ("[DUMMY] disconnecting");
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_DISCONNECTING);
-	g_usleep (G_USEC_PER_SEC * 2);
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_IDLE);
-}
-
-void
-phonemgr_listener_queue_message (PhonemgrListener *listener,
-        const char *number, const char *message)
-{
-	g_message ("[DUMMY] sending message to %s: %s", number, message);
-}
-
 /* We poll every 50 milliseconds, so this is the number of times
  * we need to go through poll() to wait for one second */
 #define POLL_SECOND 1000 / 50
@@ -773,11 +911,26 @@ phonemgr_listener_poll (PhonemgrListener *l)
 	static int i = 0;
 	static int target = 5 * POLL_SECOND;
 
+	if (l->connected == FALSE)
+		return FALSE;
+
 	i++;
 	if (i == target) {
-		char *sender = "+01234 567-89";
+		char *sender;
 		GTimeVal time;
-		char *text = "This is my test, this is my supa test";
+		char *text;
+		GRand *rand;
+
+		rand = g_rand_new_with_seed (i);
+
+		if (g_rand_boolean (rand)) {
+			sender = "+09876 543-21";
+			text = "This is my other t\303\251st, this is NOT a supa test";
+		} else {
+			sender = "+01234 567-89";
+			text = "This is my test, this is my supa test with i18n chars: \303\270\317\216\307\252";
+		}
+		g_rand_free (rand);
 
 		g_get_current_time (&time);
 
@@ -791,6 +944,37 @@ phonemgr_listener_poll (PhonemgrListener *l)
 	return TRUE;
 }
 
+gboolean
+phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **err)
+{
+	g_message ("[DUMMY] connecting to %s", device);
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTING);
+	g_usleep (G_USEC_PER_SEC * 2);
+	l->connected = TRUE;
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTED);
+
+	g_timeout_add (50, (GSourceFunc) phonemgr_listener_poll, l);
+
+	return TRUE;
+}
+
+void
+phonemgr_listener_disconnect (PhonemgrListener *l)
+{
+	g_message ("[DUMMY] disconnecting");
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_DISCONNECTING);
+	l->connected = FALSE;
+	g_usleep (G_USEC_PER_SEC * 2);
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_IDLE);
+}
+
+void
+phonemgr_listener_queue_message (PhonemgrListener *listener,
+        const char *number, const char *message)
+{
+	g_message ("[DUMMY] sending message to %s: %s", number, message);
+}
+
 gboolean
 phonemgr_listener_connected (PhonemgrListener *listener)
 {
@@ -800,7 +984,6 @@ phonemgr_listener_connected (PhonemgrListener *listener)
 static void
 phonemgr_listener_init (PhonemgrListener *l)
 {
-	g_timeout_add (50, (GSourceFunc) phonemgr_listener_poll, l);
 }
 
 static void
